-------------------------------------------------------------------------------
-- Title       : Library component declarations
-- Project     : VHDL Library of Arithmetic Units
-------------------------------------------------------------------------------
-- File        : arith_lib.vhd
-- Author      : Reto Zimmermann  <zimmi@iis.ee.ethz.ch>
-- Company     : Integrated Systems Laboratory, ETH Zurich
-- Date        : 1997/11/04
-- Version     : 1.0
-------------------------------------------------------------------------------
-- Copyright (c) 1998 Integrated Systems Laboratory, ETH Zurich
-------------------------------------------------------------------------------
-- Description :
-- Package containing all component declarations.
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
library arith_lib;
use arith_lib.arith_utils.all;

package arith_lib is

  -----------------------------------------------------------------------------
  -- Adders
  -----------------------------------------------------------------------------
  component Add
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  S : out std_logic_vector(width-1 downto 0));
  end component;

  component AddC
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component AddV
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  V : out std_logic);
  end component;

  component AddCfast
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component AddCendaround
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
          CI : in std_logic;
          S : out std_logic_vector(width-1 downto 0);
          CO : out std_logic);
  end component;

  component AddMod2Nm1
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  S : out std_logic_vector(width-1 downto 0));
  end component;

  component AddMod2Nm1s0
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  S : out std_logic_vector(width-1 downto 0));
  end component;

  component AddMod2Np1
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  S : out std_logic_vector(width-1 downto 0));
  end component;

  component AddCsv
    generic (width : integer);
    port (A1, A2, A3 : in std_logic_vector(width-1 downto 0);
	  S, C : out std_logic_vector(width-1 downto 0));
  end component;

  component AddMopCsv
    generic (width : integer;
	     depth : integer;
	     speed : integer);
    port (A : in std_logic_vector(depth*width-1 downto 0);
	  S, C : out std_logic_vector(width-1 downto 0));
  end component;

  component AddMop
    generic (width : integer;
	     depth : integer;
	     speed : integer);
    port (A : in std_logic_vector(depth*width-1 downto 0);
	  S : out std_logic_vector(width-1 downto 0));
  end component;

  -----------------------------------------------------------------------------
  -- Subtractors
  -----------------------------------------------------------------------------
  component Sub
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  S : out std_logic_vector(width-1 downto 0));
  end component;

  component SubC
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component SubCZ
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  CO, Z : out std_logic);
  end component;

  component SubV
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  V : out std_logic);
  end component;

  component SubVZ
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  V, Z : out std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- Adder/Subtractors
  -----------------------------------------------------------------------------
  component AddSub
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
          SUB : in std_logic;
	  S : out std_logic_vector(width-1 downto 0));
  end component;

  component AddSubC
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
          SUB : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component AddSubV
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
          SUB : in std_logic;
	  S : out std_logic_vector(width-1 downto 0);
	  V : out std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- Incrementers and Decrementers
  -----------------------------------------------------------------------------
  component Inc
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component IncGray
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component IncC
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  Z : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component IncGrayC
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component Dec
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component DecC
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  Z : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component IncDec
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
          DEC : in std_logic;
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component IncDecC
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
          DEC : in std_logic;
	  Z : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component Neg
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component NegC
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Neg : in std_logic;
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component AbsVal
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  -----------------------------------------------------------------------------
  -- Multipliers
  -----------------------------------------------------------------------------
  component MulUns
    generic (widthX : integer;
	     widthY : integer;
	     speed : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  P : out std_logic_vector(widthX+widthY-1 downto 0));
  end component;

  component MulSgn
    generic (widthX : integer;
	     widthY : integer;
	     speed : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  P : out std_logic_vector(widthX+widthY-1 downto 0));
  end component;

  component MulAddUns
    generic (widthX : integer;
	     widthY : integer;
	     widthA : integer;
	     speed : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  A : in std_logic_vector(widthA-1 downto 0);
	  P : out std_logic_vector(widthA-1 downto 0));
  end component;

  component MulAddSgn
    generic (widthX : integer;
	     widthY : integer;
	     widthA : integer;
	     speed : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  A : in std_logic_vector(widthA-1 downto 0);
	  P : out std_logic_vector(widthA-1 downto 0));
  end component;

  component AddMulUns
    generic (widthX : integer;
	     widthY : integer;
	     speed : integer);
    port (XS, XC : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  P : out std_logic_vector(widthX+widthY-1 downto 0));
  end component;

  component AddMulSgn
    generic (widthX : integer;
	     widthY : integer;
	     speed : integer);
    port (XS, XC : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  P : out std_logic_vector(widthX+widthY-1 downto 0));
  end component;

  component MulCsvUns
    generic (widthX : integer;
	     widthY : integer;
	     speed : integer);
    port (XS, XC : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  PS, PC : out std_logic_vector(widthX+widthY-1 downto 0));
  end component;

  component MulCsvSgn
    generic (widthX : integer;
	     widthY : integer;
	     speed : integer);
    port (XS, XC : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  PS, PC : out std_logic_vector(widthX+widthY-1 downto 0));
  end component;

  component SqrUns
    generic (width : integer;
	     speed : integer);
    port (X : in std_logic_vector(width-1 downto 0);
	  P : out std_logic_vector(2*width-1 downto 0));
  end component;

  component SqrSgn
    generic (width : integer;
	     speed : integer);
    port (X : in std_logic_vector(width-1 downto 0);
	  P : out std_logic_vector(2*width-1 downto 0));
  end component;

  -----------------------------------------------------------------------------
  -- Dividers / Square Root Extractors
  -----------------------------------------------------------------------------
  component DivArrUns
    generic (widthX : integer;
	     widthY : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  Q : out std_logic_vector(widthX-widthY downto 0);
	  R : out std_logic_vector(widthY-1 downto 0));
  end component;

  component SqrtArrUns
    generic (widthX : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Q : out std_logic_vector((widthX+1)/2-1 downto 0);
	  R : out std_logic_vector((widthX+1)/2-1 downto 0));
  end component;

  -----------------------------------------------------------------------------
  -- Miscellaneous
  -----------------------------------------------------------------------------
  component Bin2Gray
    generic (width : integer);
    port (B : in std_logic_vector(width-1 downto 0);
	  G : out std_logic_vector(width-1 downto 0));
  end component;
  
  component Gray2Bin
    generic (width : integer;
	     speed : integer);
    port (G : in std_logic_vector(width-1 downto 0);
	  B : out std_logic_vector(width-1 downto 0));
  end component;
  
  component CmpEQ
    generic (width : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  EQ : out std_logic);
  end component;
  
  component CmpGE
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  GE : out std_logic);
  end component;
  
  component CmpEQGE
    generic (width : integer;
	     speed : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  EQ, GE : out std_logic);
  end component;
  
  component AllZeroDet
    generic (width : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic);
  end component;
  
  component AllOneDet
    generic (width : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic);
  end component;
  
  component LeadZeroDet
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;
  
  component LeadOneDet
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;
  
  component LeadSignDet
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;
  
  component SumZeroDet
    generic (width : integer);
    port (A, B : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  Z : out std_logic);
  end component;
  
--compass compile_off
  component Log2
    generic (width : integer;
	     speed : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(log2ceil(width)-1 downto 0));
  end component;
--compass compile_on

  -----------------------------------------------------------------------------
  -- Subcomponents
  -----------------------------------------------------------------------------
  component PrefixAndOr
    generic (width : integer;
	     speed : integer);
    port (GI, PI : in std_logic_vector(width-1 downto 0);
	  GO, PO : out std_logic_vector(width-1 downto 0));
  end component;

  component PrefixAndOrCfast
    generic (width : integer;
	     speed : integer);
    port (GI, PI : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  GO, PO : out std_logic_vector(width-1 downto 0));
  end component;

  component PrefixAndOrCendaround
    generic (width : integer;
	     speed : integer);
    port (GI, PI : in std_logic_vector(width-1 downto 0);
	  CI : in std_logic;
	  GO, PO : out std_logic_vector(width-1 downto 0);
	  CO : out std_logic);
  end component;

  component PrefixAnd
    generic (width : integer;
	     speed : integer);
    port (PI : in std_logic_vector(width-1 downto 0);
	  PO : out std_logic_vector(width-1 downto 0));
  end component;

  component PrefixOr
    generic (width : integer;
	     speed : integer);
    port (PI : in std_logic_vector(width-1 downto 0);
	  PO : out std_logic_vector(width-1 downto 0));
  end component;

  component PrefixXor
    generic (width : integer;
	     speed : integer);
    port (PI : in std_logic_vector(width-1 downto 0);
	  PO : out std_logic_vector(width-1 downto 0));
  end component;


  component Cpr
    generic (depth : integer;
	     speed : integer);
    port (A : in std_logic_vector(depth-1 downto 0);
          CI : in std_logic_vector(depth-4 downto 0);
          S, C : out std_logic;
          CO : out std_logic_vector(depth-4 downto 0));
  end component;

--compass compile_off
  component Cnt
    generic (depth : integer;
	     speed : integer);
    port (A : in std_logic_vector(depth-1 downto 0);
          S : out std_logic_vector(log2floor(depth) downto 0));
  end component;
--compass compile_on

  component CntSlice
    generic (depth : integer;
	     speed : integer);
    port (A : in std_logic_vector(depth-1 downto 0);
          S : out std_logic;
          CO : out std_logic_vector(depth/2-1 downto 0));
  end component;


  component AddMod2Np1Csv
    generic (width : integer;
             depth : integer);
    port (A : std_logic_vector(depth*width-1 downto 0);
          S, C : out std_logic_vector(width-1 downto 0));
  end component;


  component MulPPGenUns
    generic (widthX : integer;
	     widthY : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  PP : out std_logic_vector(widthX*(widthX+widthY)-1 downto 0));
  end component;

  component MulPPGenSgn
    generic (widthX : integer;
	     widthY : integer);
    port (X : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  PP : out std_logic_vector((widthX+2)*(widthX+widthY)-1 downto 0));
  end component;

  component AddMulPPGenUns
    generic (widthX : integer;
	     widthY : integer);
    port (XS, XC : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  PP : out std_logic_vector(widthX*(widthX+widthY)-1 downto 0));
  end component;

  component AddMulPPGenSgn
    generic (widthX : integer;
	     widthY : integer);
    port (XS, XC : in std_logic_vector(widthX-1 downto 0);
	  Y : in std_logic_vector(widthY-1 downto 0);
	  PP : out std_logic_vector((widthX+1)*(widthX+widthY)-1 downto 0));
  end component;

  component SqrPPGenUns
    generic (width : integer);
    port (X : in std_logic_vector(width-1 downto 0);
	  PP : out std_logic_vector((width/2+1)*2*width-1 downto 0));
  end component;

  component SqrPPGenSgn
    generic (width : integer);
    port (X : in std_logic_vector(width-1 downto 0);
	  PP : out std_logic_vector((width/2+1)*2*width-1 downto 0));
  end component;


  -- compass compile_off
  component Decode
    generic (width : integer);
    port (A : in std_logic_vector(log2ceil(width)-1 downto 0);
	  Z : out std_logic_vector(width-1 downto 0));
  end component;

  component Encode
    generic (width : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic_vector(log2ceil(width)-1 downto 0));
  end component;
  -- compass compile_on

  
  component RedAnd
    generic (width : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic);
  end component;

  component RedOr
    generic (width : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic);
  end component;

  component RedXor
    generic (width : integer);
    port (A : in std_logic_vector(width-1 downto 0);
	  Z : out std_logic);
  end component;


  component Reg
    generic (width : integer);
    port (CLK : in std_logic;
	  RST : in std_logic;
          D : in std_logic_vector(width-1 downto 0);
          Q : out std_logic_vector(width-1 downto 0));
  end component;

  component FullAdder
    port (A, B, CI : in std_logic;
	  S, CO : out std_logic);
  end component;

  component ad01d1
    port (A, B, CI : in std_logic;
	  S, CO : out std_logic);
  end component;

end arith_lib;

-------------------------------------------------------------------------------

